@@ -113,7 +113,7 @@ def skip_line(
                 break
             char_index += 1
 
-    if ";" in line and needs_import:
+    if ";" in line.split("#")[0] and needs_import:
         for part in (part.strip() for part in line.split(";")):
             if (
                 part
@@ -200,11 +200,27 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         if skipping_line:
             out_lines.append(line)
             continue
-
-        for line in (
-            (line.strip() for line in line.split(";")) if ";" in line else (line,)  # type: ignore
+        elif (
+            config.float_to_top
+            and import_index == -1
+            and line
+            and not in_quote
+            and not line.strip().startswith("#")
         ):
-            line, raw_line = _normalize_line(line)
+            import_index = index - 1
+            while import_index and not in_lines[import_index - 1]:
+                import_index -= 1
+
+        line, *end_of_line_comment = line.split("#", 1)
+        if ";" in line:
+            statements = [line.strip() for line in line.split(";")]
+        else:
+            statements = [line]
+        if end_of_line_comment:
+            statements[-1] = f"{statements[-1]}#{end_of_line_comment[0]}"
+
+        for statement in statements:
+            line, raw_line = _normalize_line(statement)
             type_of_import = import_type(line, config) or ""
             if not type_of_import:
                 out_lines.append(raw_line)
@@ -216,15 +232,10 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
             import_string, comment = parse_comments(line)
             comments = [comment] if comment else []
             line_parts = [part for part in _strip_syntax(import_string).strip().split(" ") if part]
-            if (
-                type_of_import == "from"
-                and len(line_parts) == 2
-                and line_parts[1] != "*"
-                and comments
-            ):
+            if type_of_import == "from" and len(line_parts) == 2 and comments:
                 nested_comments[line_parts[-1]] = comments[0]
 
-            if "(" in line.split("#")[0] and index < line_count:
+            if "(" in line.split("#", 1)[0] and index < line_count:
                 while not line.split("#")[0].strip().endswith(")") and index < line_count:
                     line, new_comment = parse_comments(in_lines[index])
                     index += 1
@@ -234,7 +245,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                     if (
                         type_of_import == "from"
                         and stripped_line
-                        and " " not in stripped_line
+                        and " " not in stripped_line.replace(" as ", "")
                         and new_comment
                     ):
                         nested_comments[stripped_line] = comments[-1]
@@ -256,7 +267,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         if (
                             type_of_import == "from"
                             and stripped_line
-                            and " " not in stripped_line
+                            and " " not in stripped_line.replace(" as ", "")
                             and new_comment
                         ):
                             nested_comments[stripped_line] = comments[-1]
@@ -271,7 +282,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                             if (
                                 type_of_import == "from"
                                 and stripped_line
-                                and " " not in stripped_line
+                                and " " not in stripped_line.replace(" as ", "")
                                 and new_comment
                             ):
                                 nested_comments[stripped_line] = comments[-1]
@@ -281,7 +292,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                     if (
                         type_of_import == "from"
                         and stripped_line
-                        and " " not in stripped_line
+                        and " " not in stripped_line.replace(" as ", "")
                         and new_comment
                     ):
                         nested_comments[stripped_line] = comments[-1]
@@ -317,18 +328,30 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                 for item in _strip_syntax(import_string).split()
             ]
             straight_import = True
+            attach_comments_to: Optional[List[Any]] = None
             if "as" in just_imports and (just_imports.index("as") + 1) < len(just_imports):
                 straight_import = False
                 while "as" in just_imports:
+                    nested_module = None
                     as_index = just_imports.index("as")
                     if type_of_import == "from":
                         nested_module = just_imports[as_index - 1]
-                        module = just_imports[0] + "." + nested_module
+                        top_level_module = just_imports[0]
+                        module = top_level_module + "." + nested_module
                         as_name = just_imports[as_index + 1]
                         if nested_module == as_name and config.remove_redundant_aliases:
                             pass
                         elif as_name not in as_map["from"][module]:
                             as_map["from"][module].append(as_name)
+
+                        full_name = f"{nested_module} as {as_name}"
+                        associated_comment = nested_comments.get(full_name)
+                        if associated_comment:
+                            categorized_comments["nested"].setdefault(top_level_module, {})[
+                                full_name
+                            ] = associated_comment
+                            if associated_comment in comments:
+                                comments.pop(comments.index(associated_comment))
                     else:
                         module = just_imports[as_index - 1]
                         as_name = just_imports[as_index + 1]
@@ -336,10 +359,18 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                             pass
                         elif as_name not in as_map["straight"][module]:
                             as_map["straight"][module].append(as_name)
-                    if not config.combine_as_imports:
-                        categorized_comments["straight"][module] = comments
-                        comments = []
+
+                    if comments and attach_comments_to is None:
+                        if nested_module and config.combine_as_imports:
+                            attach_comments_to = categorized_comments["from"].setdefault(
+                                f"{top_level_module}.__combined_as__", []
+                            )
+                        else:
+                            attach_comments_to = categorized_comments["straight"].setdefault(
+                                module, []
+                            )
                     del just_imports[as_index : as_index + 2]
+
             if type_of_import == "from":
                 import_from = just_imports.pop(0)
                 placed_module = finder(import_from)
@@ -359,8 +390,9 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         ] = associated_comment
                         if associated_comment in comments:
                             comments.pop(comments.index(associated_comment))
-                if comments:
-                    categorized_comments["from"].setdefault(import_from, []).extend(comments)
+
+                if comments and attach_comments_to is None:
+                    attach_comments_to = categorized_comments["from"].setdefault(import_from, [])
 
                 if len(out_lines) > max(import_index, 1) - 1:
                     last = out_lines and out_lines[-1].rstrip() or ""
@@ -394,7 +426,14 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         (module, straight_import | root[import_from].get(module, False))
                         for module in just_imports
                     )
+
+                if comments and attach_comments_to is not None:
+                    attach_comments_to.extend(comments)
             else:
+                if comments and attach_comments_to is not None:
+                    attach_comments_to.extend(comments)
+                    comments = []
+
                 for module in just_imports:
                     if comments:
                         categorized_comments["straight"][module] = comments
